/******************************************************************************
 *                                                                            *
 * File:   nbname.c           Version  1.03            Date: 2010-04-03       *
 *                                                                            *
 * Copyright (C) 1998-2010 by Kostis Netzwerkberatung                         *
 * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056                        *
 * http://www.kostis.net/                                                     *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *        - display local NetBIOS name                                        *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     2010-04-03: KK V1.03                                                   *
 *        - recompiled after corrections in netbios.h/.lib                    *
 *     2010-03-21: KK V1.02                                                   *
 *        - added -d (debug)                                                  *
 *     2009-03-07: KK V1.01                                                   *
 *        - made source available                                             *
 *     1998-05-04: KK V1.00                                                   *
 *        - initial coding                                                    *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <netbios.h>

#include "nbname.h"

/******************************************************************************

	Function:
		display copyright message and program information

	Parameter:
		FILE		*Fout		stdout oder stderr

	Returns:
		nothing

 *****************************************************************************/

static  void    Hello
(
	FILE		*Fout
)
{
	fprintf (Fout, MSG_COPYRIGHT) ;
	fprintf (Fout, MSG_VERSION, PRGNAM, VERSION) ;
}

/******************************************************************************

	Function Variable Values

 *****************************************************************************/

#define	SCANFLAG_YES		1
#define	SCANFLAG_NO		0

/******************************************************************************

	Function Return Values

 *****************************************************************************/

#define SCANFLAG_OK		0	/* options all valid		*/
#define SCANFLAG_UNKNOWN	1	/* unknown option		*/
#define SCANFLAG_BADARGS	2	/* illegal option (value)	*/

/******************************************************************************

	Global Variables

 *****************************************************************************/

int	lana	= 0 ;
int	scan	= SCANFLAG_NO ;
int	batch	= SCANFLAG_NO ;
int	debug	= SCANFLAG_NO ;

/******************************************************************************

	Function:
		parse options

	Parameter:
		int		argc		# of parameters
		char		*argv	[]	parameters

	Returns:
		SCANFLAG_OK			all parameters valid

		SCANFLAG_UNKNOWN		unknown option
		SCANFLAG_BADARGS		illegal option (value)

 *****************************************************************************/

int	ScanFlag
(
	int		argc,
	char		*argv	[]
)
{
	int		i ;

	/**********************************************************************

		scan options

	 *********************************************************************/

	for (i = 1 ; i < argc ; ++i)
	{
		/**************************************************************

			is it a flag/switch (*NIX style)

		 *************************************************************/

		if ((*argv [i] == '-') || (*argv [i] == '/'))
		{
			/******************************************************

				which flag?

			 *****************************************************/

			switch (argv [i][1])
			{
				case	'0':
				case	'1':
				case	'2':
				case	'3':
				case	'4':
				case	'5':
				case	'6':
				case	'7':
				case	'8':
				case	'9':

					lana = atoi (&argv [i][1]) ;
					if ((lana > 255) || (lana < 0))
						return (SCANFLAG_BADARGS) ;
					break ;

				case	'b':
					batch = SCANFLAG_YES ;
					break ;

				case	'd':
					debug = SCANFLAG_YES ;
					break ;

				case	's':
					scan = SCANFLAG_YES ;
					break ;

				default:        /* unknown flag		*/

					return (SCANFLAG_UNKNOWN) ;
			}
		}
		else
		{
			/******************************************************

				no flag, that's an error

			 *****************************************************/

			return (SCANFLAG_BADARGS) ;
		}
	}

	return (SCANFLAG_OK) ;
}

/******************************************************************************

	main programm

 *****************************************************************************/

int	main
(
	int		argc,
	char		*argv	[]
)
{
	PNCB		pncb ;
	PADAPTER_STATUS	pADAPTER_STATUS ;
	PADAPTER_ALL	pADAPTER_ALL ;

	int		i ;
	char		*Help ;
	int		lana_e ;

	/**********************************************************************

		check syntax

	 *********************************************************************/

	if (ScanFlag (argc, argv) != SCANFLAG_OK)
	{
		Hello (stdout) ;
		fprintf (stdout, ERR_MSG_SYNTAX, PRGNAM) ;
		return (ERR_SYNTAX) ;
	}

	/**********************************************************************

		fetch memory for data structures

	 *********************************************************************/

	if (debug == SCANFLAG_YES)
	{
		printf ("DEBUG: fetch memory for data structures\n") ;
	}

	pncb = (PNCB) malloc (sizeof (NCB)) ;
	if (pncb == (PNCB) NULL)
		return (ERR_OUT_OF_MEMORY) ;

	pADAPTER_STATUS = (PADAPTER_STATUS) malloc (sizeof (ADAPTER_ALL)) ;
	if (pADAPTER_STATUS == (PADAPTER_STATUS) NULL)
		return (ERR_OUT_OF_MEMORY) ;

	pADAPTER_ALL = (PADAPTER_ALL) pADAPTER_STATUS ;

	/**********************************************************************

		initialize/setup data structures

	 *********************************************************************/

	if (debug == SCANFLAG_YES)
	{
		printf ("DEBUG: initialize/setup data structures\n") ;
	}

	lana_e = lana ;
	if (scan == SCANFLAG_YES)
	{
		lana_e = 255 ;
	}

	ClearNCB (pncb) ;
	ClearADAPTER_STATUS (pADAPTER_STATUS) ;

	if (debug == SCANFLAG_YES)
	{
		printf ("DEBUG: entering loop through lana\n") ;
	}

	do
	{
		if (debug == SCANFLAG_YES)
		{
			printf ("DEBUG: call NetBIOS NCBASTAT for lana %d\n", lana) ;
		}

		pncb -> ncb_command  = NCBASTAT ;
		_fmemcpy (pncb -> ncb_callname, "*", 1) ;
		pncb -> ncb_lana_num = lana ;
		pncb -> ncb_buffer   = (PUCHAR) pADAPTER_STATUS ;
		pncb -> ncb_length   = sizeof (ADAPTER_ALL) ;

		/**************************************************************

			make NetBIOS call and check results

		 *************************************************************/

		Netbios (pncb) ;

		if (debug == SCANFLAG_YES)
		{
			printf ("DEBUG: NetBIOS returned 0x%02x\n", pncb -> ncb_retcode) ;
		}


		if (pncb -> ncb_retcode != NRC_GOODRET)
		{
			++lana ;
		}
	}
	while ((pncb -> ncb_retcode != NRC_GOODRET) && (lana <= lana_e)) ;

	/**********************************************************************

		if NetBIOS call was successful, display result

	 *********************************************************************/
	
	if (pncb -> ncb_retcode == NRC_GOODRET)
	{
		if (pADAPTER_STATUS -> name_count > 0)
		{
			if (batch == SCANFLAG_YES)
			{
				fprintf (stdout, "set NBNAME=") ;
			}

			Help = pADAPTER_ALL -> names [0].name ;
			for (i = 0 ; (*Help != ' ') && (i < NCBNAMSZ - 1) ; ++Help, ++i)
				fprintf (stdout, "%c", *Help) ;
			fprintf (stdout, "\n") ;
		}
		else
		{
			Hello (stdout) ;
			fprintf (stdout, "%s: NetBIOS returned zero name length for lana %d\n", PRGNAM, lana) ;
		}
	}

	/**********************************************************************

		free memory for data structures

	 *********************************************************************/

	free ((void *) pADAPTER_STATUS) ;
	free ((void *) pncb) ;

	return (ERR_NONE) ;
}
